/**
 * Copyright (C) 2008  Bull S. A. S.
 * Bull, Rue Jean Jaures, B.P.68, 78340, Les Clayes-sous-Bois
 * This library is free software; you can redistribute it and/or modify it under the terms
 * of the GNU Lesser General Public License as published by the Free Software Foundation
 * version 2.1 of the License.
 * This library is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU Lesser General Public License for more details.
 * You should have received a copy of the GNU Lesser General Public License along with this
 * program; if not, write to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA  02110-1301, USA.
 **/

package org.ow2.orchestra.console.security;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Map;

import org.exoplatform.container.PortalContainer;
import org.exoplatform.services.organization.Membership;
import org.exoplatform.services.organization.OrganizationService;

/**
 * The profile manager allow to manage the user associated profiles. It map the
 * couple of user membership and user group with a set of access profiles.
 * 
 * @author Elwalid Brahim <elwalid.brahim@bull.net>
 * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
 */

public class ProfileManager {

  private Map<String, Collection<String>> profilesMap;

  public ProfileManager() {
    profilesMap = new HashMap<String, Collection<String>>();
  }

  /**
   * Add a profile to a group or membership
   * 
   * @param id a group membership
   * @param profile a profile
   * @author Elwalid Brahim <elwalid.brahim@bull.net>
   * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
   */
  public void addProfile(String id, String profile) {
    if (!this.profilesMap.containsKey(id)) {
      this.profilesMap.put(id, new HashSet<String>());
    }
    this.profilesMap.get(id).add(profile);
  }

  /**
   * add profiles to a group or membership
   * 
   * @param id a group membership
   * @param profiles a set of profile
   * @author Elwalid Brahim <elwalid.brahim@bull.net>
   * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
   * @todo check if this method is usefull
   */
  public void addProfiles(String id, Collection<String> profiles) {
    if (!this.profilesMap.containsKey(id)) {
      this.profilesMap.put(id, profiles);
    } else {
      this.profilesMap.get(id).addAll(profiles);
    }
  }

  /**
   * remove profiles from a group or membership
   * 
   * @param id a group membership
   * @param profiles a set of profile
   * @author Elwalid Brahim <elwalid.brahim@bull.net>
   * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
   * @todo check if this method is usefull
   */
  public void removeProfiles(String id, Collection<String> profiles) {
    if (this.profilesMap.containsKey(id)) {
      this.profilesMap.get(id).removeAll(profiles);
    }
  }

  /**
   * get the profiles for a given user
   * 
   * @param userName the user name
   * @return the profiles for the specified user
   * @author Elwalid Brahim <elwalid.brahim@bull.net>
   * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
   */
  @SuppressWarnings("unchecked")
  public Collection<String> getProfiles(String userName) {
    PortalContainer portalContainer = PortalContainer.getInstance();
    OrganizationService organisationService = 
      (OrganizationService) portalContainer.getComponentInstanceOfType(OrganizationService.class);

    Collection<Membership> userMemberships = null;
    try {
      userMemberships = organisationService.getMembershipHandler().findMembershipsByUser(userName);
    } catch (Exception e) {
      System.err.println("Impossible to get the memberships list");
    }
    Collection<String> profiles = new HashSet<String>();
    if (userMemberships != null) {
      Iterator<Membership> iter = userMemberships.iterator();
      String id;
      // for all groups we get the associated profiles
      while (iter.hasNext()) {
        Membership membership = iter.next();
        id = membership.getMembershipType() + ":" + membership.getGroupId();
        if (this.profilesMap.get(id) != null) {
          profiles.addAll(this.profilesMap.get(id));
        }
      }
    }
    return profiles;
  }

  /**
   * get the profiles with the given profile id
   * 
   * @param id group}:{membership}
   * @return the profiles for the specified id
   * @author Elwalid Brahim <elwalid.brahim@bull.net>
   * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
   */
  public Collection<String> getProfilesById(String id) {
    return this.profilesMap.get(id);
  }

  /**
   * commit the Profile Manager in the JCR
   * 
   * @author Elwalid Brahim <elwalid.brahim@bull.net>
   * @author Le Gall Rodrigue <rodrigue.le-gall@bull.net>
   * @todo check if this method is usefull
   */
  public void commit() {
    // store on the jcr
  }
}
